﻿using System;
using System.Collections.Concurrent;

namespace KS.Foundation
{
	public class ObjectPool<T>
	{
		readonly ConcurrentBag<T> _objects;
		Func<T> _objectGenerator;

		public ObjectPool(Func<T> objectGenerator)
		{
			if (objectGenerator == null) throw new ArgumentNullException("objectGenerator");
			_objects = new ConcurrentBag<T>();
			_objectGenerator = objectGenerator;
		}

		public T GetObject()
		{
			T item;
			if (_objects.TryTake(out item)) return item;
			return _objectGenerator();
		}

		public void PutObject(T item)
		{
			_objects.Add(item);
		}
	}
}

